// QFormField.h: interface for the QFormField class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(QFORMFIELD_H__INCLUDED_)
#define QFORMFIELD_H__INCLUDED_

#include "stdafx.h"
#include "Oidtable.h"
#include "qutil.h"

// Forward references
class QSearchClass;

////////////////////////////////////////////////////////////////////////////////
// A field within an input form. Each field is described bythe field label
// and the list of attribute types used to generate a search filter.
class AFX_EXT_CLASS QFormField
{
public:
	QFormField( CString &field_name, CString &default_value, PtrList<QAttrT> &filt_attr_list );
	~QFormField();

public:
  QFormField *CopySelf();
	CString &Name() { return field_name ; } ;
	CString &Value() { return input_value ; } ;
	BOOL GenerateFilterString( CString &filter_str );

private:
	CString field_name;
	CString input_value;

  // The attributes that this filter component will match on. If the attributes
  // are cn and sn then the resulting filter component would be:
  //
  // &(cn=<search-string>)(sn=<search-string>)
  //
  // i.e. the attribute filter components OR'ed 
	PtrList<QAttrT> filt_attrt_list;
};

////////////////////////////////////////////////////////////////////////////////
// Specification of an input within a form. An input consists of one or more
// edit input fields (QFormField) that correspond to a given entry type. For
// example, the search input for a search class "Person" may consist of the
// input fields "Name", "E-Mail Address" and "Job Title".
class AFX_EXT_CLASS QFormInput
{
public:
  QFormInput( CString &input_name, QSearchClass *srchclass, PtrList<QFormField> &field_list );
  ~QFormInput();

public:
  QFormInput *CopySelf();
	UINT GetFieldNames( QStringList &field_names );
	BOOL SetFieldValue( CString &field_name, CString &field_val );

public:
  CString &Name() { return input_name ; } ;
  QSearchClass *SearchClass() { return srchclass ; } ;
  BOOL GenerateFilterString( CString &filt_str );

private:
  // Input label.
  CString input_name;

	// List of fields that make up this input form component
  PtrList<QFormField> field_list;

  // Pointer to the search class that this is a search input form for
  QSearchClass *srchclass;
};

#endif // QFORMFIELD_H__INCLUDED_