// QFormSearchNode.h: interface for the QFormSearchNode class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QFORMSEARCHNODE_H__12F4A0A1_F999_11D2_89E9_B4096BE2201B__INCLUDED_)
#define AFX_QFORMSEARCHNODE_H__12F4A0A1_F999_11D2_89E9_B4096BE2201B__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "stdafx.h"
#include "QFormOp.h"
#include "QDSManager.h"
#include "QFormField.h"
#include "QDN.h"

// A node in the search "graph". Each node represents a stage in the search, where state
// is represented by:
//
// 1. The form field inputs remaining (fi_left_list below).
// 2. The target form field (fi below). This is constant across all nodes in a particular
//    search request.
// 3. Entries found so far using consumed search criteria (i.e. form field inputs already used).
class QFormSearchNode
{
public:
  QFormSearchNode
    ( QDSManager *ds,
			long conn_id,
			QForm *form,
      QFormInput *ffi_target,
			PtrList<QFormInput> &ffi_left_list,
      QDN &base_dn,
			CString &filter_str );

  virtual ~QFormSearchNode();

public:
  // Invoke a search operation using the given search specification (return LDAP message ID)
  BOOL Invoke( QError *&err ) { return op->Invoke( err ) ; } ;

  // Abort the operation in progress
  void Abort() { if ( op != NULL ) op->Abort() ; } ;

  // Get matches. Matches may be returned asynchronously.
  UINT GetMatches( PtrList<QEntry> &match_list ) { return op->GetMatches( match_list ) ; } ;

  // Get returned referrals. Referrals may be returned asynchronously.
  UINT GetReferrals( QStringList &ref_list ) { return op->GetReferrals( ref_list ) ; } ;

	// 

public:
	// Has the search operation been invoked?
	BOOL Invoked() { return op->Invoked() ; } ;

	// Search operation complete?
	BOOL Done()	{ return op->Done() ; } ;

	// Search operation completed without error?
	BOOL Succeeded() { return op->Succeeded() ; } ;

  // Get associated LDAP context
  LDAP *GetLDAPContext() { return op->GetLDAPContext() ; } ;

	// Get associated connection ID
	long GetConnID() { return op->GetConnID() ; } ;

	// Does this node represent a 'leaf' search?
	BOOL TargetSearch() { return ( ffi_target == NULL ? TRUE : FALSE ) ; } ;

public:
  // Handle an LDAP message 
  BOOL HandleLDAPMessage( LDAP *ld, LDAPMessage *lm, QError *& err )
    { return op->HandleLDAPMessage( ld, lm, err ) ; } ;

  // Does this operation have the given message ID?
  BOOL IsTaskOwner( LDAP *ld, int ldap_msg_id )
		{ return op->IsTaskOwner( ld, ldap_msg_id ) ; } ;

public:
	BOOL GetRemainingInput
		( QFormInput **target_ffi_ptr, PtrList<QFormInput> *left_ffi_ptr );

private:
  // The form input specifying the target entry
  QFormInput *ffi_target;

  // List of unprocessed inputs 
  PtrList<QFormInput> ffi_left_list;

private:
	QFormOp *op;
};

template PtrList<QFormSearchNode>;

#endif // !defined(AFX_QFORMSEARCHNODE_H__12F4A0A1_F999_11D2_89E9_B4096BE2201B__INCLUDED_)
