// QRequestManager.cpp: implementation of the QRequestManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Query32.h"
#include "QRequestManager.h"
#include <limits.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

QRequestManager::QRequestManager()
{
  req_id_acc = 0;
  polling = FALSE;
}

QRequestManager::~QRequestManager()
{
  POSITION pos = task_list.GetHeadPosition();
  while (pos != NULL)
    {
      task->Abort();
      delete task;
    }
}

void QRequestManager::Poll()
{
  // No work to do?
  if (request_list.GetCount() == 0)
    return;

  // Make sure that we don't re-enter.
  if (polling == TRUE)
    return;

  polling = TRUE;

  struct timeval timeout;
  BOOLEAN done = FALSE, left;
  int task_id, lerr, req_id;
  QTask task;
  LDAPMessage *lm;

  timeout.tv_sec = 0;
  timeout.tv_usec = 0;

  // Poll on each open connection
  POSITION pos = bind_mgr.conn_map.GetStartPosition();
  while (pos != NULL)
    {
      long conn_id;
      QConnection *conn;
      
      // Get the next connection
      bind_mgr.conn_map.GetNextAssoc( pos, conn_id, conn );

      do
        {
          lerr = ldap_result( conn->GetLDAPContext(), LDAP_RES_ANY, 1, &timeout, &lm );
          if (lerr == 0)
	          // No responses on this connection
	          {
	            PGunlock(h_task);
	            break;
	          }
  
          // If things get this far then a response has been received
          QRequest *req = GetTaskOwner( lm->ldap_msgid() );
          if (req != NULL)
	          {
    	        req->HandleLDAPMessage( ds, lm );
    	        ldap_msgfree( lm );
	          }
        }
      // Continue polling on this connection until there is no more pending data
      while (Waiting() && conn->Select( &timeout ) > 0)
    }
      
  // All done
  polling = FALSE;
}

// Register a newly invoked request
QRequestID QRequestManager::StartRequest( QRequest *requester )
{
  QRequestID new_id = req_id_acc;
  if (req_id_acc == ULONG_MAX)
    req_id_acc = 0;

  QRequest *req = new QRequest( this, new_id, requester );
  request_list.AddTail( req );

  return new_id;
}

// Notify of request completion
void QRequestManager::FinishRequest( QRequestID req_id )
{
  POSITION pos = request_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QRequest *req = request_list.GetNext( pos );
      if (req->ID() == req_id)
        {
          request_list.RemoveAt( pos );
          delete req;
        }
    }
}

// Get a request record
QRequest *QRequestManager::GetRequest( QRequestID req_id )
{
  POSITION pos = request_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QRequest *req = request_list.GetNext( pos );
      if (req->ID() == req_id)
        return req;
    }

  return NULL;
}

void QRequestManager::AbortRequest(QRequestID req_id)
{
  POSITION pos = request_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QRequest *req = request_list.GetNext( pos );
      if (req->ID() == req_id)
        {
          request_list.RemoveAt( pos );
          req->Abort();
          delete req;
        }
    }
}

void QRequestManager::AbortAll()
{
  if ( request_list.GetCount() == 0 )
    return;

  POSITION pos = request_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QRequest *req = request_list.GetNext( pos );
      req->Abort();
      delete req;
    }

  request_list.RemoveAll();
}
