// QSearchClass.h: interface for the QSearchClass class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(QSEARCHCLASS_H__INCLUDED_)
#define QSEARCHCLASS_H__INCLUDED_

#include "ptrlist.h"
#include "QDN.h"

// Forward references
class QOC;
class QAttr;
class QDSManager;

class AFX_EXT_CLASS QSearchClass
{
public:
  // Use when class is identified by object class values and is searchable
  QSearchClass
    ( Oidtable *table, CString &name, PtrList<QOC> &oc_list );

  // Used when class is specific and represents a named entry
  QSearchClass
    ( QDSManager *ds,
			CString &name,
			CString &dn,
			CString &host,
			int port,
			CString &user,
			CString &password );

	// Used when class is specific and represents a named entry
	QSearchClass( QDSManager *ds, CString &name, CString &ldap_uri );

  ~QSearchClass();

public:
  void AddSearchClass( QSearchClass *sc );
  UINT GetSearchClassList( PtrList<QSearchClass> &class_list );

public:
  const CString &Name() { return name ; } ;
  BOOL HasName( const CString &name ) { return this->name.CompareNoCase( name ) ; } ;

public:
  BOOL IsSpecific( QDN &dn_ref );
  BOOL CanSearchFor( QSearchClass *sc );

public:
  BOOL GenerateFilterString( CString &oc_filt_str);
  BOOL GetDN( QDN &get_dn );
  BOOL HasObjectClass( QOC *oc );
	BOOL HasObjectClass( QAttr *oc_attr );

private:
  // Class is a named entry (specific) or is identified by object class (general)
  BOOL is_specific;

  // DN and host details (if a specific class)
  QDN dn;
  CString host;
  int port;
  CString username;
  CString password;

  // User friendly name for this class, e.g. "Person"
  CString name;

	// Object classes that identify entries of this search class
	PtrList<QOC> oc_list;

  // Types of entry that can be searched for...
  PtrList<QSearchClass> srch_class_list;

	// Oidtable in use
	Oidtable *table;
};

#endif
